package org.tests.event;

import org.jbpm.workflow.instance.impl.WorkflowProcessInstanceImpl;
import org.kie.api.event.process.DefaultProcessEventListener;
import org.kie.api.event.process.ProcessNodeTriggeredEvent;
import org.kie.api.event.process.ProcessStartedEvent;
import org.kie.api.runtime.KieSession;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.tests.model.WorkflowCase;

public class MyProcessEventListener extends DefaultProcessEventListener {

    private final Logger logger = LoggerFactory.getLogger(MyProcessEventListener.class);

    private static final String WF_NODE_LOAD = "Load Case";
    private static final String VAR_CASE_ID = "caseId";
    private static final String VAR_WF_CASE = "wfCase";

    @Override
    public void beforeProcessStarted(ProcessStartedEvent event) {
        logger.debug("Load data for process {} with id {}", event.getProcessInstance().getProcessId(), event.getProcessInstance().getId());

        // Check caseId
        WorkflowProcessInstanceImpl pi = (WorkflowProcessInstanceImpl) event.getProcessInstance();

        // don't load case for sub process
        if (pi.getParentProcessInstanceId() <= 0) {
            loadCase((KieSession) event.getKieRuntime(), pi);
        }
    }

    @Override
    public void beforeNodeTriggered(ProcessNodeTriggeredEvent event) {

        if (WF_NODE_LOAD.equals(event.getNodeInstance().getNodeName())) {
            logger.debug("Load data for process {} with id {}", event.getProcessInstance().getProcessId(), event.getProcessInstance().getId());

            // TODO load data form external database
            // Check caseId
            WorkflowProcessInstanceImpl pi = (WorkflowProcessInstanceImpl) event.getProcessInstance();

            // don't load case for sub process
            if (pi.getParentProcessInstanceId() <= 0) {
                loadCase((KieSession) event.getKieRuntime(), pi);
            }

        }

    }

    private void loadCase(KieSession ksession, WorkflowProcessInstanceImpl pi) {
        String caseFolderId = (String) pi.getVariable(VAR_CASE_ID);

        if (caseFolderId == null || caseFolderId.equals("")) {
            throw new IllegalStateException("Can not find caseId in process variable.");
        }

        WorkflowCase wfCaseBefore = (WorkflowCase) pi.getVariable(VAR_WF_CASE);
        logger.debug("Case before load: " + wfCaseBefore);

        // load case form external data
        WorkflowCase wfCase = new WorkflowCase();
        wfCase.setCaseId("case-id");
        wfCase.setCaseOrigin("case origin");
        wfCase.setServiceLevel("level");

        // Update WorlflowCase in process instance
        // Doesn't works
        pi.setVariable(VAR_WF_CASE, wfCase);

        // Doesn't works
        // VariableScopeInstance variableScopeInstance = (VariableScopeInstance)
        // pi.getContextInstance(VariableScope.VARIABLE_SCOPE);
        // if (variableScopeInstance != null) {
        // variableScopeInstance.setVariable(VAR_WF_CASE, wfCase);
        // }

        // Doesn't works
        // Map<String, Object> variables = new HashMap<String, Object>(1);
        // variables.put(VAR_WF_CASE, wfCase);
        // ksession.execute(new SetProcessInstanceVariablesCommand(pi.getId(), variables));

        WorkflowCase wfCaseAfter = (WorkflowCase) pi.getVariable(VAR_WF_CASE);
        logger.debug("Case after load: " + wfCaseAfter);
    }

}
