package org.tests.bpmn;

import java.util.HashMap;
import java.util.Map;

import org.jbpm.bpmn2.handler.ReceiveTaskHandler;
import org.jbpm.process.workitem.bpmn2.ServiceTaskHandler;
import org.jbpm.test.JbpmJUnitBaseTestCase;
import org.junit.Test;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.manager.RuntimeEngine;
import org.kie.api.runtime.process.ProcessInstance;
import org.kie.api.task.TaskService;
import org.kie.api.task.model.TaskSummary;
import org.tests.event.MyProcessEventListener;

/**
 * JUnit tests for subscription process.
 */
public class SubscriptionProcessTest extends JbpmJUnitBaseTestCase {


    private static final String SUBSCRIPTON_PROCESS = "org/tests/bpmn/subscription.bpmn";
    private static final String PROCESS_NAME = "subscription";

    private static final String TASK_CASE_EVALUATION = "CaseEvaluation";
    private static final String TASK_CASE_EVALUATION_OK = "OK";
    private static final String TASK_CASE_EVALUATION_KO = "KO";

    private static final String USER_ID = "john";

    /**
     * Call super class constructor with in memory persistance config.
     */
    public SubscriptionProcessTest() {
        super(true, true);

    }

    /**
     * Tests case "OK".
     */
    @Test
    public void testCaseComplete() {
        createRuntimeManager(SUBSCRIPTON_PROCESS);
        RuntimeEngine engine = getRuntimeEngine();
        KieSession ksession = engine.getKieSession();
        ksession.getWorkItemManager().registerWorkItemHandler("Service Task", new ServiceTaskHandler(ksession));
        ReceiveTaskHandler receiveTaskHandler = new ReceiveTaskHandler(ksession);
        ksession.getWorkItemManager().registerWorkItemHandler("Receive Task", receiveTaskHandler);
        ksession.addEventListener(new MyProcessEventListener());

        // Start process
        ProcessInstance processInstance = startProcessForManualCase(ksession);

        // Case evaluation task
        TaskService taskService = engine.getTaskService();
        TaskSummary userChoiceTask = taskService.getTasksAssignedAsPotentialOwner("john", "en").get(0);
        assertEquals(userChoiceTask.getName(), TASK_CASE_EVALUATION);
        taskService.start(userChoiceTask.getId(), USER_ID);

        // Case evaluation task complete
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("taskCaseEvaluation", TASK_CASE_EVALUATION_OK);
        taskService.complete(userChoiceTask.getId(), USER_ID, params);
        assertNodeTriggered(processInstance.getId(), TASK_CASE_EVALUATION);

        assertProcessInstanceCompleted(processInstance.getId());

        System.out.println("Process " + PROCESS_NAME + " case complete test done.");
    }

    private ProcessInstance startProcessForManualCase(KieSession ksession) {
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("caseId", "case-manual-identifier");
        ProcessInstance processInstance = ksession.startProcess(PROCESS_NAME, params);

        assertProcessInstanceActive(processInstance.getId());
        return processInstance;
    }

}
